@@ -28,6 +28,10 @@
  * See the ChangeLog files for a list of changes. 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs.h>
 
@@ -63,8 +67,6 @@ struct _GeditDocumentPrivate
 	GeditUndoManager *undo_manager;
 };
 
-static gint current_max_untitled_num = 0;
-
 enum {
 	NAME_CHANGED,
 	SAVED,
@@ -102,6 +104,52 @@ static gboolean gedit_document_auto_save_timeout (GeditDocument *doc);
 static GtkTextBufferClass *parent_class 	= NULL;
 static guint document_signals[LAST_SIGNAL] 	= { 0 };
 
+static GHashTable* allocated_untitled_numbers = NULL;
+
+static gint gedit_document_get_untitled_number (void);
+static void gedit_document_release_untitled_number (gint n);
+
+static gint
+gedit_document_get_untitled_number (void)
+{
+	gint i = 1;
+	
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	if (allocated_untitled_numbers == NULL)
+		allocated_untitled_numbers = g_hash_table_new (NULL, NULL);
+
+	g_return_val_if_fail (allocated_untitled_numbers != NULL, -1);
+	
+	while (TRUE)
+	{
+		if (g_hash_table_lookup (allocated_untitled_numbers, GINT_TO_POINTER (i)) == NULL)
+		{
+			g_hash_table_insert (allocated_untitled_numbers, 
+					     GINT_TO_POINTER (i),
+					     GINT_TO_POINTER (i));
+	
+			return i;
+		}
+		
+		++i;
+	}					
+}
+
+static void
+gedit_document_release_untitled_number (gint n)
+{
+	gboolean ret;
+	
+	g_return_if_fail (allocated_untitled_numbers != NULL);
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	ret = g_hash_table_remove (allocated_untitled_numbers, GINT_TO_POINTER (n));
+	g_return_if_fail (ret);	
+}
+
+
 
 GType
 gedit_document_get_type (void)
@@ -279,20 +327,19 @@ gedit_document_finalize (GObject *object)
 	if (document->priv->auto_save_timeout > 0)
 		g_source_remove (document->priv->auto_save_timeout);
 
+	if (document->priv->untitled_number > 0)
+	{
+		g_return_if_fail (document->priv->uri == NULL);
+		gedit_document_release_untitled_number (
+				document->priv->untitled_number);
+	}
+
 	if (document->priv->uri)
     	{
 		g_free (document->priv->uri);
       		document->priv->uri = NULL;
-
-		if (current_max_untitled_num == document->priv->untitled_number)
-			--current_max_untitled_num;
-    	}
-	else
-	{
-		if (current_max_untitled_num == document->priv->untitled_number)
-			--current_max_untitled_num;	
-	}
-
+	}		
+	
 	if (document->priv->last_searched_text)
 		g_free (document->priv->last_searched_text);
 
@@ -324,7 +371,9 @@ gedit_document_new (void)
 	document = GEDIT_DOCUMENT (g_object_new (GEDIT_TYPE_DOCUMENT, NULL));
 
 	g_return_val_if_fail (document->priv != NULL, NULL);
-  	document->priv->untitled_number = ++current_max_untitled_num;
+  	
+	document->priv->untitled_number = gedit_document_get_untitled_number ();
+	g_return_val_if_fail (document->priv->untitled_number > 0, NULL);
 
 	return document;
 }
@@ -334,9 +383,9 @@ gedit_document_new (void)
  * @uri: the URI of the file that has to be loaded
  * @error: return location for error or NULL
  * 
- * Creates a new untitled document.
+ * Creates a new document.
  *
- * Return value: a new untitled document
+ * Return value: a new document
  **/
 GeditDocument*
 gedit_document_new_with_uri (const gchar *uri, GError **error)
@@ -594,8 +643,9 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 				if (conv_error != NULL)
 					g_error_free (conv_error);
 
-				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_WRONG_FORMAT,
-				_("Invalid UTF-8 data encountered reading file"));
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, 
+					     GEDIT_ERROR_INVALID_UTF8_DATA,
+					     _("Invalid UTF-8 data"));
 				
 				if (converted_file_contents != NULL)
 					g_free (converted_file_contents);
@@ -700,8 +750,8 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	{
 		if (!g_utf8_validate (file_contents->str, file_contents->len, NULL))
 		{
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_WRONG_FORMAT,
-				_("Invalid UTF-8 data encountered reading file"));
+			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GEDIT_ERROR_INVALID_UTF8_DATA,
+				_("Invalid UTF-8 data"));
 			g_string_free (file_contents, TRUE);
 			return FALSE;
 		}
@@ -758,7 +808,12 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 		g_free (doc->priv->uri);
 			
 	doc->priv->uri = g_strdup (uri);
-	doc->priv->untitled_number = 0;
+
+	if (doc->priv->untitled_number > 0)
+	{
+		gedit_document_release_untitled_number (doc->priv->untitled_number);
+		doc->priv->untitled_number = 0;
+	}
 	
 	g_signal_emit (G_OBJECT (doc), document_signals[NAME_CHANGED], 0);
 }
@@ -859,14 +914,20 @@ gedit_document_save_a_copy_as (GeditDocument* doc, const gchar *uri, GError **er
 	return ret;
 }
 
+/* FIXME: define new ERROR_CODE and remove the error 
+ * strings from here -- Paolo
+ */
+#define BAK_EXTENSION_PLUS "~00~"
+
 static gboolean	
 gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	       gboolean create_backup_copy, GError **error)
 {
-	FILE* file;
-	gchar* chars;
-	gchar* fname;
-	gchar* bak_fname = NULL;
+	FILE  *file;
+	gchar *chars;
+	gchar *fname;
+	gchar *bak_fname = NULL;
+	gchar *temp_fname = NULL;
 	gboolean have_backup = FALSE;
 	gboolean res = FALSE;
 	
@@ -878,30 +939,43 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 
 	if (!gedit_utils_uri_has_file_scheme (uri))
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, EROFS, g_strerror (EROFS));
-		return FALSE;
-	
+		gchar *error_message;
+		gchar *scheme_string = gnome_vfs_x_uri_get_scheme (uri);
+                
+		if (scheme_string != NULL)
+		{
+			error_message = g_strdup_printf (
+				_("gedit cannot handle %s: locations in write mode."),
+                               	scheme_string);
+
+			g_free (scheme_string);
+		}
+		else
+			error_message = g_strdup_printf (
+				_("gedit cannot handle this kind of location in write mode."));
+
+		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, EROFS, error_message);
+		return FALSE;	
 	}
 	
 	fname = gnome_vfs_x_format_uri_for_display (uri);
 
-	if (create_backup_copy && (fname != NULL))
+	if (fname != NULL)
 	{
-		bak_fname = g_strconcat (fname, gedit_settings->backup_extension, NULL);
+		temp_fname = g_strconcat (fname, gedit_settings->backup_extension, BAK_EXTENSION_PLUS, NULL);
 		
-		if ((bak_fname == NULL) || (rename (fname, bak_fname) != 0))
+		if ((temp_fname == NULL) || (rename (fname, temp_fname) != 0))
     		{
       			if (errno != ENOENT)
 			{
 				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
-					"Cannot create the backup copy of the file.\n"
-					"The file has not been saved.");
+					_("Could not create the backup copy of the file."));
 		
 				if (fname != NULL) 
 					g_free (fname);
 
-				if (bak_fname != NULL) 
-					g_free (bak_fname);
+				if (temp_fname != NULL) 
+					g_free (temp_fname);
 
 				return FALSE;
 
@@ -909,13 +983,13 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
     		}
   		else
     			have_backup = TRUE;
-	}
+	}	
 
 	if ((fname == NULL) || ((file = fopen (fname, "w")) == NULL))
 	{
 		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
-			"Make sure that the path you provided exists,\n"
-			"and that you have the appropriate write permissions.");
+			_("Make sure that the path you provided exists, "
+			  "and that you have the appropriate write permissions."));
 						
 		res = FALSE;
 
@@ -946,7 +1020,24 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	
 	if (fputs (chars, file) == EOF || fclose (file) == EOF)
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno, g_strerror (errno));
+		gchar *errmsg;
+
+		switch (errno)
+		{
+			case ENOSPC: 
+				errmsg = g_strdup_printf (
+					_("There is no enough disk space to save the file.\n"
+					  "Please, free some space on the disk and try again."));
+				break;
+			case EFBIG:
+				errmsg = g_strdup_printf (
+					_("The file is too big."));
+				break;
+			default:
+				errmsg = g_strdup_printf (" "); 
+		}
+
+		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno, errmsg);
 		g_free (chars);
 
 		res = FALSE;
@@ -961,13 +1052,28 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	res = TRUE;
 	
 finally:
-	if (!res && have_backup)
-		/* FIXME: should the errors be managed ? - Paolo */
-      		rename (bak_fname, fname);
+
+	if (have_backup)
+	{
+		/* FIXME: should the I/O errors be managed ? - Paolo */
+		if (!res)
+      			rename (temp_fname, fname);
+		else 
+			if (create_backup_copy)
+			{
+				bak_fname = g_strconcat (fname, gedit_settings->backup_extension, NULL);
+				rename (temp_fname, bak_fname);
+			}
+			else
+				unlink (temp_fname);			
+	}
 	
 	if (fname != NULL) 
 		g_free (fname);
 
+	if (temp_fname != NULL) 
+		g_free (temp_fname);
+
 	if (bak_fname != NULL) 
 		g_free (bak_fname);
 
@@ -1074,7 +1180,7 @@ gedit_document_revert (GeditDocument *doc,  GError **error)
 
 	if (gedit_document_is_untitled (doc))
 	{
-		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GNOME_VFS_ERROR_GENERIC,
+		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GEDIT_ERROR_UNTITLED,
 			_("It is not possible to revert an Untitled document"));
 		return FALSE;
 	}
@@ -1447,6 +1553,12 @@ gedit_document_get_selected_text (GeditDocument *doc, gint *start, gint *end)
 	if (end != NULL)
 		*end = gtk_text_iter_get_offset (&sel_bound); 
 
+	if (gtk_text_iter_equal (&sel_bound, &iter))
+	{
+		gedit_debug (DEBUG_DOCUMENT, "There is no selected text");
+
+		return NULL;
+	}
 	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &iter, &sel_bound, TRUE);
 }
 
@@ -1483,6 +1595,7 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (replace != NULL);
+	g_return_if_fail (*replace != '\0');
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
                                     &iter,
@@ -1493,7 +1606,14 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
                                     &sel_bound,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					                      "selection_bound"));
-		
+
+	if (gtk_text_iter_equal (&sel_bound, &iter))
+	{
+		gedit_debug (DEBUG_DOCUMENT, "There is no selected text");
+
+		return;
+	}
+	
 	gtk_text_iter_order (&sel_bound, &iter);		
 
 	gedit_document_begin_user_action (doc);
@@ -1502,11 +1622,16 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 				&iter,
 				&sel_bound);
 
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
+                                    &iter,
+                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					                      "insert"));
+
 	gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc),
 				&iter,
 				replace, strlen (replace));
 
-	if (doc->priv->last_replace_text)
+	if (doc->priv->last_replace_text != NULL)
 		g_free (doc->priv->last_replace_text);
 
 	doc->priv->last_replace_text = g_strdup (replace);
